from __future__ import annotations

from dataclasses import dataclass

from sziszapangma.core.alignment.step_type import StepType
from sziszapangma.core.alignment.step_words import StepWords


@dataclass(frozen=True)
class AlignmentProcessingStep:
    step_type: StepType
    step_words: StepWords
    previous_distance: float
    step_cost: float

    @classmethod
    def levenshtein_insertion(
        cls, previous_distance: float, step_words: StepWords, step_cost: float = 1
    ) -> AlignmentProcessingStep:
        words = StepWords(None, step_words.hypothesis_word)
        return AlignmentProcessingStep(StepType.INSERTION, words, previous_distance, step_cost)

    @classmethod
    def levenshtein_deletion(
        cls, previous_distance: float, step_words: StepWords, step_cost: float = 1
    ) -> AlignmentProcessingStep:
        words = StepWords(step_words.reference_word, None)
        return AlignmentProcessingStep(StepType.DELETION, words, previous_distance, step_cost)

    @classmethod
    def levenshtein_substitution(
        cls, previous_distance: float, step_words: StepWords, step_cost: float
    ) -> AlignmentProcessingStep:
        return AlignmentProcessingStep(
            StepType.SUBSTITUTION, step_words, previous_distance, step_cost
        )

    @classmethod
    def levenshtein_correct(
        cls, previous_distance: float, step_words: StepWords, step_cost: float
    ) -> AlignmentProcessingStep:
        return AlignmentProcessingStep(StepType.CORRECT, step_words, previous_distance, step_cost)

    def total_distance(self) -> float:
        return self.step_cost + self.previous_distance
