from typing import Dict, List

import numpy as np
import numpy.typing as npt

from sziszapangma.core.transformer.embedding_transformer import EmbeddingTransformer


class CachedEmbeddingTransformer(EmbeddingTransformer):
    _embeddings_transformer: EmbeddingTransformer
    _cache: Dict[str, npt.NDArray[np.float64]]

    def __init__(self, embeddings_transformer: EmbeddingTransformer):
        self._embeddings_transformer = embeddings_transformer
        self._cache = dict()

    def get_embedding(self, word: str) -> npt.NDArray[np.float64]:
        return self.get_embeddings([word])[word]

    def get_embeddings(self, words: List[str]) -> Dict[str, npt.NDArray[np.float64]]:
        new_words = [word for word in words if word not in self._cache]
        new_embeddings = (
            self._embeddings_transformer.get_embeddings(new_words) if len(new_words) > 0 else dict()
        )
        for new_word in new_words:
            self._cache[new_word] = new_embeddings[new_word]
        return {word: self._cache[word] for word in words}

    def clear(self) -> None:
        self._cache.clear()
