from typing import Dict, List

import fasttext
import fasttext.util
import numpy as np
import numpy.typing as npt

from sziszapangma.core.transformer.embedding_transformer import EmbeddingTransformer


class FasttextEmbeddingTransformer(EmbeddingTransformer):

    _model_name: str

    def __init__(self, lang_id: str):
        full_model_name = fasttext.util.download_model(lang_id, if_exists="ignore")
        self._fasttext_model = fasttext.load_model(full_model_name)

    def get_embedding(self, word: str) -> npt.NDArray[np.float64]:
        return np.array(self._fasttext_model.get_word_vector(word))

    def get_embeddings(self, words: List[str]) -> Dict[str, npt.NDArray[np.float64]]:
        return {word: self.get_embedding(word) for word in set(words)}
