import json
from typing import Dict, List, Optional

import numpy as np
import requests

from sziszapangma.core.transformer.embedding_transformer import EmbeddingTransformer


class WebEmbeddingTransformer(EmbeddingTransformer):
    _lang_id: str
    _host: str
    _auth_token: Optional[str]

    def __init__(self, lang_id: str, host: str, auth_token: Optional[str]):
        self._lang_id = lang_id
        self._host = host
        self._auth_token = auth_token

    def get_embedding(self, word: str) -> np.ndarray:
        return self._make_request([word])[word]

    def get_embeddings(self, words: List[str]) -> Dict[str, np.ndarray]:
        return self._make_request(words)

    def _make_request(self, words: List[str]) -> Dict[str, np.ndarray]:
        print(f"call for words: {json.dumps(words)}")
        headers = {"Content-Type": "application/json"}
        if self._auth_token is not None:
            headers["Authorization"] = f"Bearer {self._auth_token}"
        result = requests.post(
            f"{self._host}/process_embedding/{self._lang_id}", headers=headers, json=words
        )
        if not result.ok:
            print(f"response_code {result.status_code}")
        result_text = result.text
        parsed_result = json.loads(result_text)
        return {it: np.array(parsed_result[it]) for it in parsed_result.keys()}
