from abc import ABC, abstractmethod
from pathlib import Path
from typing import List, Optional


class PathFilter(ABC):
    """
    Abstract class of PathFilter.

    Filter provide all audio files to preprocess by ASR and calculate metrics.
    """

    @abstractmethod
    def get_list_of_files(self) -> List[str]:
        """Returns list of audio files."""


class ExtensionPathFilter(PathFilter):
    """
    Implementation of PathFilter which find all files with specified extension.
    """

    _extension: str
    _root_directory: str
    _files_limit: Optional[int]

    def __init__(self, root_directory: str, extension: str, files_limit: Optional[int] = None):
        """Constructor of class."""
        self._extension = extension
        self._files_limit = files_limit
        self._root_directory = root_directory

    def get_list_of_files(self) -> List[str]:
        """
        Implementation of searching files with extension.
        """
        path_generator = Path(self._root_directory).glob(f"**/*.{self._extension}")
        all_files = [str(it) for it in path_generator]
        return all_files if self._files_limit is None else all_files[: self._files_limit]
