"""Repository to manage results of asr experiment processing."""
from abc import ABC, abstractmethod
from typing import Any, Optional, Set


class ExperimentRepository(ABC):
    """Repository to manage results of asr experiment processing."""

    @abstractmethod
    def initialise(self) -> None:
        """Method to initialize repository."""

    @abstractmethod
    def property_exists(self, record_id: str, property_name: str) -> bool:
        """Method checks that property in record exists."""

    @abstractmethod
    def update_property_for_key(self, record_id: str, property_name: str, property_value: Any):
        """Method updates property in record."""

    @abstractmethod
    def delete_property_for_key(self, record_id: str, property_name: str):
        """Method removes property in record."""

    @abstractmethod
    def get_property_for_key(self, record_id: str, property_name: str) -> Optional[Any]:
        """Method returns property for record."""

    @abstractmethod
    def get_metrics_result_to_df(self, metrics_property: str):
        pass

    @abstractmethod
    def get_all_record_ids(self) -> Set[str]:
        """Methods returns all possible record ids."""

    @abstractmethod
    def get_all_properties(self) -> Set[str]:
        """Methods returns all possible properties."""
