from typing import Any, Optional, Set

from pymongo import MongoClient
from pymongo.database import Database

from sziszapangma.integration.repository.experiment_repository import ExperimentRepository

ID = "_id"
VALUE = "value"


class MongoExperimentRepository(ExperimentRepository):
    _mongo_client: MongoClient
    _database_name: str

    def __init__(self, mongo_client: MongoClient, database_name: str):
        super().__init__()
        self._mongo_client = mongo_client
        self._database_name = database_name

    def initialise(self):
        return

    def property_exists(self, record_id: str, property_name: str) -> bool:
        database = self._get_database()
        all_collections = database.list_collection_names()
        if property_name not in all_collections:
            return False
        else:
            return database[property_name].find_one({ID: record_id}) is not None

    def update_property_for_key(self, record_id: str, property_name: str, property_value: Any):
        self.delete_property_for_key(record_id, property_name)
        self._get_database()[property_name].insert_one({ID: record_id, VALUE: property_value})

    def delete_property_for_key(self, record_id: str, property_name: str):
        if self.property_exists(record_id, property_name):
            self._get_database()[property_name].delete_one({ID: record_id})

    def get_property_for_key(self, record_id: str, property_name: str) -> Optional[Any]:
        if self.property_exists(record_id, property_name):
            return self._get_database()[property_name].find_one({ID: record_id})[VALUE]
        else:
            return None

    def get_metrics_result_to_df(self, metrics_property: str):
        # TODO: implement
        pass

    def _get_database(self) -> Database:
        return self._mongo_client[self._database_name]

    def get_all_record_ids(self) -> Set[str]:
        records = set()
        for collection_name in self.get_all_properties():
            for record in self._get_database()[collection_name].find():
                records.add(record[ID])
        return records

    def get_all_properties(self) -> Set[str]:
        return set(self._get_database().list_collection_names())
